/*
*  InputHandler.cpp: by Gary Pack and Luke Tokheim. September 23, 2002.
*/
#include "InputHandler.h"

using namespace ambenv;

InputHandler::InputHandler( std::ifstream & fileIn ) : m_lineNumber( 0 )
{
    fin = &fileIn;    
}

InputHandler::~InputHandler()
{
  if( fin ) {
    fin = NULL;
  }
}

const std::vector<std::string> & InputHandler::getArgs()
{
  return( vargs );
}

unsigned InputHandler::getLineNumber() const
{
  return( m_lineNumber );
}

int InputHandler::getLine()
{
  int retval = 1;
  unsigned long chars = 0;
  char ch = 0;

  // this is a new line so make sure to clear the string vector
  vargs.clear();

  // create the first string in the vector
  std::string s;
  vargs.push_back( s );

  while( 1 ) {
    // -- make sure we do hit EOF
    if( !fin->get( ch ) ) {
      retval = 0;
      break;
    }

    // -- This is the begin of a comment, advance to the next newline.
    if( '#' == ch ) {
      bool eofflag = true;
      while( fin->get( ch ) ) {
	if( '\n' == ch ) {
	  eofflag = false;
	  break;
	}
      }
      if( eofflag ) {
	retval = 0;
	break;
      }
    }

    // -- If we hit a null character, squash it out.
    if( ch == '\0' ) {
      continue;
    }
    if( isWhitespace( ch ) ) {
      // -- push new string onto the end of the vector of strings	
      assert( !vargs.empty() );
      if( (vargs.back()).size() > 0 ) {
	vargs.push_back( s );
      }
    } else {
      if( '\n' == ch) {
	m_lineNumber++;
	// -- if there is an empty line, just skip over it
	if( chars < 1 ) {
	  continue;
	}
	break;
      } else {
	// -- push ch onto end of current (last in vector) string
	assert( vargs.size() );
	chars++;
	(vargs.back()).append( &ch, 1 ); /* For stupid-ass MSVC6 compiler from hell */
      }
    }
  }
  
  return( retval );
}

bool InputHandler::isWhitespace( const char & ch )
{
    if((' ' == ch) || ('\t' == ch)) {
    	return true;
    } else {
   	return false; 
    }
}

bool InputHandler::isComment( const char & ch )
{
  if( '#' == ch ) {
    return( true );
  } else {
    return( false );
  }
}
