/*
*  InputHandler.h: by Gary Pack and Luke Tokheim. December 12, 2002.
*
*	Read a line from the file input of arbitrary length n
*	terminated by the '\n' character and tokenize by whitespace 
*	into the member vector of strings. Squashes out '\0' bytes.
*
*       Consider '#' to be comment and ingore the rest of the line.
*       BROKEN: Don't try and comment a line with "data" tokens on it. 
*/
#ifndef __INPUT_HANDLER_H_
#define __INPUT_HANDLER_H_

#include <cassert>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

namespace ambenv {

  class InputHandler {
  public:
    InputHandler( std::ifstream & fileIn );
    ~InputHandler();

    // Read a line from standard input of length n
    // and tokenize by whitespace into a vector of
    // strings. If the null character is encountered
    // it is not added to the current token, but is
    // only skipped.
    // Returns 0 at EOF, else returns 1.
    int getLine();
  
    // return reference to the member variable
    const std::vector<std::string> & getArgs();

    unsigned getLineNumber() const;

  private:
    std::vector<std::string> vargs;
    std::ifstream * fin;
    unsigned m_lineNumber;

    // return TRUE if ch is '\t' or ' '
    bool isWhitespace( const char & ch );
    bool isComment( const char & ch );
  };

}; /* namespace ambenv */

#endif /* __INPUT_HANDLER_H_ */
