#include "Instance.h"

#include <cassert>

using namespace ambenv;

Instance::Instance() : m_object( NULL )
{
}

Instance::Instance( const std::string & name, const std::string & material, 
		    const Object * object )
  : m_name( name ),  m_material( material ), m_object( object )
{
}

// -- get accessors
std::string Instance::name() const
{
  return( m_name );
}

const Object * Instance::object() const
{
  return( m_object );
}

std::string Instance::material() const
{
  return( m_material );
}

// -- set accessors
void Instance::name( const std::string & name )
{
  m_name = name;
}

void Instance::material( const std::string & material )
{
  m_material = material;
}

bool Instance::intersect( const Ray & ray, Real epsilon ) const
{
  assert( m_object != NULL );

  Ray obj_ray( getMatrixInverse().multVec( ray.getOrigin() ),
  	 getMatrixInverse().multVec( ray.getDirection(), false ) );

  return( m_object->intersect( obj_ray, epsilon ) );

}
