#ifndef __MATERIAL_H_
#define __MATERIAL_H_

#include "Instance.h"
#include "Light.h"
#include "Nameable.h"
#include "Ray.h"
#include "Vec3.h"

#include <string>
#include <list>

namespace ambenv {

  //! Material base class. Overload shade in inherited classes to use a different light model.
  /*! Default shade class uses Phong model.
   */
  class Material : public Nameable {
  public:
    Material();
    Material( const std::string & name, const Vec3 & emission,
	      const Vec3 & ambient, const Vec3 & diffuse, const Vec3 & specular,
	      const Real specular_power, const std::string & texture );

    virtual ~Material();

    // -- get accessors
    Vec3 emission() const;
    Vec3 ambient() const;
    Vec3 diffuse() const;
    Vec3 specular() const;
    Real specular_power() const;
    std::string texture() const;
    unsigned materialId() const;

    // -- set accessors
    void emission( const Vec3 & emission );
    void ambient( const Vec3 & ambient );
    void diffuse( const Vec3 & diffuse );
    void specular( const Vec3 & specular );
    void specular_power( const Real specular_power );
    void texture( const std::string & texture );
    void materialId( unsigned id );
 
  private:
    Vec3 m_emission;
    Vec3 m_ambient;
    Vec3 m_diffuse;
    Vec3 m_specular;
    Real m_specular_power;
    std::string m_texture;
    unsigned m_materialId;
  };

}; /* namespace ambenv */

#endif /* __MATERIAL_H_ */



