#ifndef __MESH_ELEMENTS_H_
#define __MESH_ELEMENTS_H_

#include "Vec3.h"

#include <cassert>
#include <vector>
#include <list>

namespace ambenv {

  class MeshVertex {
  public:
    MeshVertex() : m_occl( 1.0 ) { }
    MeshVertex( const Vec3 & v, const Vec3 & n, const Vec3 & tc ) 
      : m_vertex( v ), m_normal( n ), m_tcoord( tc ), m_occl( 1.0 ) { }

    MeshVertex( const Vec3 & v, const Vec3 & n, const Vec3 & tc, const Vec3 & vcolor ) 
      : m_vertex( v ), m_normal( n ), m_tcoord( tc ), m_vcolor( vcolor ), m_occl( 1.0 ) { }

    Vec3 v() const { return( m_vertex ); }
    Vec3 n() const { return( m_normal ); }
    Vec3 tc() const { return( m_tcoord ); }
    Vec3 color() const { return( m_vcolor ); }
    Real occl() const { return( m_occl ); }
    Vec3 bentNormal() const { return( m_bentNormal ); }

    void v( const Vec3 & vertex ) { m_vertex = vertex; }
    void n( const Vec3 & normal ) { m_normal = normal; }
    void tc( const Vec3 & tcoord ) { m_tcoord = tcoord; }
    void color( const Vec3 & vcolor ) { m_vcolor = vcolor; }
    void occl( Real r ) { m_occl = r; }
    void bentNormal( const Vec3 & normal ) { m_bentNormal = normal; }

  private:
    Vec3 m_vertex;
    Vec3 m_normal;
    Vec3 m_tcoord;
    Vec3 m_vcolor;
    Real m_occl;
    Vec3 m_bentNormal;
  };

  class MeshEdge {
  public:
    MeshEdge() : m_vert1( -1 ), m_vert2( -1 ), m_tri1( -1 ), m_tri2( -1 ), m_marked( false ) { }
	
  private:
    int m_vert1;
    int m_vert2;
    int m_tri1;
    int m_tri2;
    
    bool m_marked;
  };
  
  class MeshTriangle {
  public:
    MeshTriangle() : m_textureId( 0 ), m_materialId( 0 )
      {
	m_vert[0] = -1;
	m_vert[1] = -1;
	m_vert[2] = -1;
      }
    int vert( unsigned i ) const { 
      assert( i < 3 ); 
      return( m_vert[i] ); 
    }
    void vert( unsigned i, int v ) {
      assert( i < 3 );
      m_vert[i] = v;
    }

    unsigned textureId() const
    {
      return( m_textureId );
    }

    void textureId( unsigned tid )
    {
      m_textureId = tid;
    }

    unsigned materialId() const
    {
      return( m_materialId );
    }

    void materialId( unsigned mid )
    {
      m_materialId = mid;
    }

  private:
    int m_vert[3];
    unsigned m_textureId;
    unsigned m_materialId;
  };

}; /* namespace ambenv */

#endif /* __MESH_ELEMENTS_H_ */
